using System;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using Plugins.UserControls;
using StockModel;
using Utils;
using Utils.Extensions;

namespace Plugins.Common
{
	public class DataFeeder : IDisposable
	{
		#region Attributes

		private bool _lastQuotation;
		private bool _forcedClosedTimePassed;

		private readonly HourSelectorData _hourSettings;

		#endregion

		#region Properties

		public ObservableCollection<Quotation> Source { get; private set; }
		public ObservableCollection2<Quotation> Data { get; private set; }

		#endregion

		#region Public Methods

		public DataFeeder(ObservableCollection<Quotation> source, HourSelectorData hourSettings)
		{
			_hourSettings = hourSettings;

			this.Source = source;
			
			this.Data = new ObservableCollection2<Quotation>();

			this.Source.CollectionChanged += this.Source_CollectionChanged; // Disposed
		}

		public void SetNextQuotationIsLastOne()
		{
			_lastQuotation = true;
		}

		#endregion

		#region Event Handlers

		private void Source_CollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
		{
			if (e.Action == NotifyCollectionChangedAction.Add)
			{
				Quotation currentQuotation = (Quotation)e.NewItems[e.NewItems.Count - 1]; // Only take the last quotation received but it will receive only one

				if (!_forcedClosedTimePassed && currentQuotation.Type != QuoteType.Closed && ((_hourSettings.StartWithOpening && currentQuotation.Type != QuoteType.OpeningBid) || (!_hourSettings.StartWithOpening && currentQuotation.Date.TimeOfDay >= _hourSettings.OperationStart)) && currentQuotation.Date.TimeOfDay <= _hourSettings.ForcedCloseTime)
				{
					if (_lastQuotation && currentQuotation.Date.TimeOfDay < _hourSettings.ForcedCloseTime)
					{
						currentQuotation.Date = currentQuotation.Date.ChangeTime(_hourSettings.ForcedCloseTime); // Small hack to force the closing of open operations when in history mode without enought data.
					}

					_forcedClosedTimePassed = currentQuotation.Date.TimeOfDay > _hourSettings.ForcedCloseTime;

					this.Data.Add(currentQuotation);
				}
			}
			else if(e.Action == NotifyCollectionChangedAction.Reset) // When the loader resets the feeder too, reset also internal state
			{
				this.Data.Clear();

				_lastQuotation = false;
				_forcedClosedTimePassed = false;
			}
		}

		#endregion

		#region IDisposable

        protected bool Disposed { get; private set; }

        private void Dispose(bool disposing)
        {
            if (!this.Disposed)
            {
                this.InternalDispose(disposing);
            }

            this.Disposed = true;
        }

        protected virtual void InternalDispose(bool disposing)
        {
            if (disposing)
            {
            	this.Source.CollectionChanged -= this.Source_CollectionChanged;
			}

            // Dispose here only unmanaged objects 
            // Dont use managed objects here because maybe 
            // they have been finalized already
        }

        public void Dispose()
        {
            this.Dispose(true);
            GC.SuppressFinalize(this);
        }

        ~DataFeeder()
        {
            this.Dispose(false);
        }

        #endregion
	}
}